#include "widget.h"
#include "autorizationdialog.h"
#include <QSharedPointer>
#include <QSql>
#include <QSqlQuery>
#include <QSqlError>
#include <QDebug>
#include <QApplication>
#include <QByteArray>
#include <QCryptographicHash>

const QString DataBaseSettings::type = "QSQLITE";
const QString DataBaseSettings::hostname = "localhost";
const QString DataBaseSettings::name = "autorization.sqlite";

const QString UsersTable::tableName = "tbl";
const QString UsersTable::id = "id";
const QString UsersTable::user = "user";
const QString UsersTable::passwordHash = "pass_hash";

Widget::Widget(QWidget *parent)
    : QWidget(parent)
{
    m_label = new QLabel("Войдите в систему", this);
    m_autorizeButton = new QPushButton("Войти", this);
    m_cancelButton = new QPushButton("Отмена", this);

    m_buttonsLayout = new QHBoxLayout;
    m_buttonsLayout->addWidget(m_autorizeButton);
    m_buttonsLayout->addWidget(m_cancelButton);

    m_mainLayout = new QVBoxLayout;
    m_mainLayout->addWidget(m_label);
    m_mainLayout->addLayout(m_buttonsLayout);

    setLayout(m_mainLayout);

    connect(m_autorizeButton, &QPushButton::clicked, this, &Widget::slotAutorize);
    connect(m_cancelButton, &QPushButton::clicked, this, &Widget::close);

    db = QSqlDatabase::addDatabase(DataBaseSettings::type);
    db.setHostName(DataBaseSettings::hostname);
    db.setDatabaseName(qApp->applicationDirPath() + "/" + DataBaseSettings::name);

    if(db.open())
        qDebug() << "DB opened";
    else
        qDebug() << "DB don't opened";

    QSqlQuery queryTable;
    queryTable.prepare(
        "CREATE TABLE " + UsersTable::tableName + " (" +
        UsersTable::id + " INTEGER PRIMARY KEY AUTOINCREMENT, " +
        UsersTable::user + " TEXT, " +
        UsersTable::passwordHash + " TEXT" +       ")"
                      );

    if(!queryTable.exec())
    {
        qDebug() << "Error create table";
        qDebug() << queryTable.lastError().text();
    }
    else
        qDebug() << "Created table";

    QSqlQuery queryInsert;
    queryInsert.prepare(
        "INSERT INTO " + UsersTable::tableName + " (" +
        UsersTable::user + ", " +
        UsersTable::passwordHash +                ") " +
        "VALUES (:User, :PassHash)"
                       );
    QString userPassword = "123456";
    QByteArray BAuserPassword = userPassword.toUtf8();
    QString userPassworHash = QCryptographicHash::hash(BAuserPassword, QCryptographicHash::Sha256).toHex();

    queryInsert.bindValue(":User", "user");
    queryInsert.bindValue(":PassHash", userPassworHash);

    if(!queryInsert.exec())
    {
        qDebug() << "Error insert new user-password into table";
        qDebug() << queryInsert.lastError().text();
    }
    else
    {
        qDebug() << "Inserted user-password into table";
    }
}

Widget::~Widget()
{
}

void Widget::slotAutorize()
{
    QSharedPointer<AutorizationDialog> dialog(new AutorizationDialog());

    if(dialog->exec() == QDialog::Accepted)
    {
        QString userFromForm = dialog->getUser();
        QString passwordFromForm = dialog->getPassword();
        QByteArray BApasswordFromForm = passwordFromForm.toUtf8();
        QString passwordHashFromForm = QCryptographicHash::hash(BApasswordFromForm, QCryptographicHash::Sha256).toHex();

        QSqlQuery querySelect;
        querySelect.prepare(
            "SELECT * FROM " + UsersTable::tableName +
            " WHERE " + UsersTable::user + " = :User"
                           );
        querySelect.bindValue(":User", userFromForm);

        if(!querySelect.exec())
        {
            qDebug() << "Error select user from table";
            qDebug() << querySelect.lastError().text();
        }
        else
        {
            qDebug() << "Selected user from table";
        }

        querySelect.next();
        QString userFromDB = querySelect.value(UsersTable::E_COLUMN_USER).toString();
        QString passwordHashFromDB = querySelect.value(UsersTable::E_COLUMN_PASSWORD_HASH).toString();

        if(userFromDB.isEmpty() || userFromForm != userFromDB)
        {
            m_label->setText("Неверный логин");
        }
        else if(passwordHashFromDB.isEmpty() || passwordHashFromForm != passwordHashFromDB)
        {
            m_label->setText("Неверный логин или пароль");
        }
        else
        {
            m_label->setText("Вы вошли в систему");
        }
    }
}
